import { IWebSocket, IRequest } from './TNode';
export declare enum WSState {
    CONNECT = "wssConnect",
    OPEN = "wssOpen",
    CLOSE = "wssClose",
    HANDSHAKE_ERROR = "wssHandshakeError",
    MISMATCH_PROTOCOL = "wssMismatchProtocol",
    ERROR = "wssError"
}
/**
 * Creates an instance of a WebSocketClient.
 * @param {IWebSocket} ws - The WebSocket object representing the connection.
*/
export declare class WebSocketClient {
    private id;
    data: string;
    path: string;
    state: WSState;
    constructor(ws: IWebSocket);
    /**
     * Changes current WebSocket state
     * @param {string} state WebSocket state from happyx bindings
     *
     * @throws {TypeError} when state is not websocket state.
     */
    setState(state: string): void;
    /**
     * Sends a text message over the WebSocket connection.
     * @param {string} text - The text message to send.
     */
    sendText(text: string): void;
    /**
     *  Sends a JSON object over the WebSocket connection.
     * @param {object} data - The JSON data to send.
     */
    sendJson(data: object): void;
    /**
     * Sends either a text message or a JSON object over the WebSocket connection based on the data type.
     * @param {string | object} data - The data to send, which can be a string or an object.
     */
    send(data: string | object): void;
    /**
    * Closes the WebSocket connection.
    */
    close(): void;
}
export declare class RequestModel {
    private requestModelObj;
    constructor(modelName: string, modelFields: {
        [key: string]: any;
    });
}
/**
 * Represents an HTTP request.
 * @param {IRequest} req - The HTTP request object.
*/
export declare class Request {
    private id;
    path: string;
    method: string;
    hostname: string;
    body: string;
    params: any;
    queries: {
        [key: string]: string;
    };
    headers: {
        [key: string]: string;
    };
    cookies: {
        [key: string]: string;
    };
    constructor(request: IRequest);
    /**
        * Sends a response to the HTTP request with optional status code and headers.
        * @param {string | number | boolean | object} data - The response data.
        * @param {number} [code=200] - The HTTP status code (default is 200).
        * @param {object} [headers] - The HTTP response headers (default is {"Content-Type": "text/plaintext"}).
     */
    answer(data: string | number | boolean | object, code?: number, headers?: object): void;
    /**
     * Sends a JSON response to the HTTP request with optional status code.
     * @param {object} data - The JSON data to send.
     * @param {number} [code=200] - The HTTP status code (default is 200).
     */
    answerJson(data: object, code?: number): void;
    /**
      * Retrieves a query parameter from the HTTP request.
     * @param {string} queryKey - The name of the query parameter.
     * @returns {string | null} The value of the query parameter or null if not found.
     */
    query(queryKey: string): string | null;
}
/**
 * Creates an instance of Server.
 * @param {string} [address="127.0.0.1"] - The IP address to bind the server to (default is "127.0.0.1").
 * @param {number} [port=5000] - The port number to listen on (default is 5000).
 *
 * @example
 * // new server at http://127.0.0.1:5000
 * const app = new Server();
 *
 * // new server at http://0.0.0.0:80
 * const app = new Server('0.0.0.0', 80);
 */
export declare class Server {
    private address;
    private port;
    private serverId;
    constructor(address?: string, port?: number, title?: string);
    /**
     * Starts the HTTP server.
     */
    start(): void;
    /**
     * Defines a GET route on the server.
     * @param {string} path - The route path.
     * @param {(request: Request) => any} callback - The callback function to handle the GET request.
     */
    get(path: string, callback: (request: Request) => any, docs?: string): void;
    /**
     * Defines a POST route on the server.
     * @param {string} path - The route path.
     * @param {(request: Request) => any} callback - The callback function to handle the POST request.
     */
    post(path: string, callback: (request: Request) => any, docs?: string): void;
    /**
     * Defines a PUT route on the server.
     * @param {string} path - The route path.
     * @param {(request: Request) => any} callback - The callback function to handle the PUT request.
     */
    put(path: string, callback: (request: Request) => any, docs?: string): void;
    /**
     * Defines a DELETE route on the server.
     * @param {string} path - The route path.
     * @param {(request: Request) => any} callback - The callback function to handle the DELETE request.
     */
    delete(path: string, callback: (request: Request) => any, docs?: string): void;
    /**
     * Defines a PATCH route on the server.
     * @param {string} path - The route path.
     * @param {(request: Request) => any} callback - The callback function to handle the PATCH request.
     */
    patch(path: string, callback: (request: Request) => any, docs?: string): void;
    /**
     * Defines a OPTIONS route on the server.
     * @param {string} path - The route path.
     * @param {(request: Request) => any} callback - The callback function to handle the OPTIONS request.
     */
    options(path: string, callback: (request: Request) => any, docs?: string): void;
    /**
     * Defines a HEAD route on the server.
     * @param {string} path - The route path.
     * @param {(request: Request) => any} callback - The callback function to handle the HEAD request.
     */
    head(path: string, callback: (request: Request) => any, docs?: string): void;
    /**
     * Defines a TRACE route on the server.
     * @param {string} path - The route path.
     * @param {(request: Request) => any} callback - The callback function to handle the TRACE request.
     */
    trace(path: string, callback: (request: Request) => any, docs?: string): void;
    /**
     * Defines a COPY route on the server.
     * @param {string} path - The route path.
     * @param {(request: Request) => any} callback - The callback function to handle the TRACE request.
     */
    copy(path: string, callback: (request: Request) => any, docs?: string): void;
    /**
     * Defines a PURGE route on the server.
     * @param {string} path - The route path.
     * @param {(request: Request) => any} callback - The callback function to handle the TRACE request.
     */
    purge(path: string, callback: (request: Request) => any, docs?: string): void;
    /**
     * Defines a middleware route on the server.
     * @param {(request: Request) => any} callback - The callback function to handle all HTTP requests.
     */
    middleware(callback: (request: Request) => any): void;
    /**
     * Defines a route on the server that touches when not any routes matched.
     * @param {(request: Request) => any} callback - The callback function to handle the 404 route.
     */
    notfound(callback: (request: Request) => any): void;
    /**
    * Defines a WebSocket route on the server.
     * @param {string} path - The route path.
     * @param {(wsClient: WebSocketClient) => any} callback - The callback function to handle WebSocket connections.
    */
    ws(path: string, callback: (wsClient: WebSocketClient) => any): void;
    /**
    * Defines a WebSocket route on the server.
     * @param {string} path - The route path.
     * @param {(wsClient: WebSocketClient) => any} callback -
     *        The callback function to handle WebSocket connections.
     *
     * @example
     *  asdasdasdasdasdsa;
     *
     */
    mount(path: string, other: Server): void;
    /**
     * Serves static files from a directory under a specific path.
     * @param {string} path - The route path.
     * @param {string} directory - The directory to serve static files from.
     */
    static(path: string, directory: string): void;
}
export declare function newPathParamType(name: string, pattern: string | RegExp, cb: (data: string) => any): void;
//# sourceMappingURL=index.d.ts.map