"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.newPathParamType = exports.Server = exports.Request = exports.RequestModel = exports.WebSocketClient = exports.WSState = void 0;
const hpx = require('./bin/node.node'); // .node is "C" instance
var WSState;
(function (WSState) {
    WSState["CONNECT"] = "wssConnect";
    WSState["OPEN"] = "wssOpen";
    WSState["CLOSE"] = "wssClose";
    WSState["HANDSHAKE_ERROR"] = "wssHandshakeError";
    WSState["MISMATCH_PROTOCOL"] = "wssMismatchProtocol";
    WSState["ERROR"] = "wssError";
})(WSState || (exports.WSState = WSState = {}));
/**
 * Creates an instance of a WebSocketClient.
 * @param {IWebSocket} ws - The WebSocket object representing the connection.
*/
class WebSocketClient {
    constructor(ws) {
        this.id = ws.websocketId;
        this.data = ws.data;
        this.path = ws.path;
        this.state = WSState.CONNECT;
        this.setState(ws.state);
    }
    /**
     * Changes current WebSocket state
     * @param {string} state WebSocket state from happyx bindings
     *
     * @throws {TypeError} when state is not websocket state.
     */
    setState(state) {
        for (const key of Object.keys(WSState)) {
            if (WSState[key] === state) {
                this.state = WSState[key];
                return;
            }
        }
        throw new TypeError(`${state} is not WebSocket state!`);
    }
    /**
     * Sends a text message over the WebSocket connection.
     * @param {string} text - The text message to send.
     */
    sendText(text) {
        hpx.hpxWebSocketSendText(this.id, text);
    }
    /**
     *  Sends a JSON object over the WebSocket connection.
     * @param {object} data - The JSON data to send.
     */
    sendJson(data) {
        hpx.hpxWebSocketSendJson(this.id, data);
    }
    /**
     * Sends either a text message or a JSON object over the WebSocket connection based on the data type.
     * @param {string | object} data - The data to send, which can be a string or an object.
     */
    send(data) {
        if (typeof data === "string") {
            hpx.hpxWebSocketSendText(this.id, data);
        }
        else {
            hpx.hpxWebSocketSendJson(this.id, data);
        }
    }
    /**
    * Closes the WebSocket connection.
    */
    close() {
        hpx.hpxWebSocketClose(this.id);
    }
}
exports.WebSocketClient = WebSocketClient;
class RequestModel {
    constructor(modelName, modelFields) {
        this.requestModelObj = {
            model: modelName,
            fields: {}
        };
        for (const field in modelFields) {
            const type = typeof modelFields[field];
            if (type === 'object' && modelFields[field].hasOwnProperty('model')) {
                this.requestModelObj.fields[field] = modelFields[field].model;
            }
            else if (type === 'function') {
                continue;
            }
            else {
                this.requestModelObj.fields[field] = typeof modelFields[field];
            }
        }
        hpx.hpxRegisterRequestModel(modelName, this.requestModelObj.fields);
    }
}
exports.RequestModel = RequestModel;
/**
 * Represents an HTTP request.
 * @param {IRequest} req - The HTTP request object.
*/
class Request {
    constructor(request) {
        this.id = request.reqId;
        this.hostname = request.hostname;
        this.method = request.method;
        this.params = request.params;
        this.path = request.path;
        this.queries = request.queries;
        this.body = request.body;
        this.headers = request.headers;
        this.cookies = request.cookies;
    }
    /**
        * Sends a response to the HTTP request with optional status code and headers.
        * @param {string | number | boolean | object} data - The response data.
        * @param {number} [code=200] - The HTTP status code (default is 200).
        * @param {object} [headers] - The HTTP response headers (default is {"Content-Type": "text/plaintext"}).
     */
    answer(data, code = 200, headers = {
        "Content-Type": "text/plaintext"
    }) {
        switch (typeof data) {
            case "number":
                hpx.hpxRequestAnswerInt(this.id, data, code, headers);
                break;
            case "string":
                hpx.hpxRequestAnswerStr(this.id, data, code, headers);
                break;
            case "boolean":
                hpx.hpxRequestAnswerBool(this.id, data, code, headers);
                break;
            case "object":
                hpx.hpxRequestAnswerObj(this.id, data, code, headers);
                break;
        }
    }
    /**
     * Sends a JSON response to the HTTP request with optional status code.
     * @param {object} data - The JSON data to send.
     * @param {number} [code=200] - The HTTP status code (default is 200).
     */
    answerJson(data, code = 200) {
        this.answer(data, code, { "Content-Type": "application/json" });
    }
    /**
      * Retrieves a query parameter from the HTTP request.
     * @param {string} queryKey - The name of the query parameter.
     * @returns {string | null} The value of the query parameter or null if not found.
     */
    query(queryKey) {
        if (queryKey in this.queries)
            return this.queries[queryKey];
        return null;
    }
}
exports.Request = Request;
/**
 * Creates an instance of Server.
 * @param {string} [address="127.0.0.1"] - The IP address to bind the server to (default is "127.0.0.1").
 * @param {number} [port=5000] - The port number to listen on (default is 5000).
 *
 * @example
 * // new server at http://127.0.0.1:5000
 * const app = new Server();
 *
 * // new server at http://0.0.0.0:80
 * const app = new Server('0.0.0.0', 80);
 */
class Server {
    constructor(address = "127.0.0.1", port = 5000, title = "HappyX NodeJS project") {
        this.address = address;
        this.port = port;
        this.serverId = hpx.hpxServer(this.address, this.port, title);
    }
    /**
     * Starts the HTTP server.
     */
    start() {
        hpx.hpxStartServer(this.serverId);
    }
    /**
     * Defines a GET route on the server.
     * @param {string} path - The route path.
     * @param {(request: Request) => any} callback - The callback function to handle the GET request.
     */
    get(path, callback, docs = "") {
        hpx.hpxServerGet(this.serverId, path, (req) => {
            return callback(new Request(req));
        }, docs);
    }
    /**
     * Defines a POST route on the server.
     * @param {string} path - The route path.
     * @param {(request: Request) => any} callback - The callback function to handle the POST request.
     */
    post(path, callback, docs = "") {
        hpx.hpxServerPost(this.serverId, path, (req) => {
            return callback(new Request(req));
        }, docs);
    }
    /**
     * Defines a PUT route on the server.
     * @param {string} path - The route path.
     * @param {(request: Request) => any} callback - The callback function to handle the PUT request.
     */
    put(path, callback, docs = "") {
        hpx.hpxServerPut(this.serverId, path, (req) => {
            return callback(new Request(req));
        }, docs);
    }
    /**
     * Defines a DELETE route on the server.
     * @param {string} path - The route path.
     * @param {(request: Request) => any} callback - The callback function to handle the DELETE request.
     */
    delete(path, callback, docs = "") {
        hpx.hpxServerDelete(this.serverId, path, (req) => {
            return callback(new Request(req));
        }, docs);
    }
    /**
     * Defines a PATCH route on the server.
     * @param {string} path - The route path.
     * @param {(request: Request) => any} callback - The callback function to handle the PATCH request.
     */
    patch(path, callback, docs = "") {
        hpx.hpxServerPatch(this.serverId, path, (req) => {
            return callback(new Request(req));
        }, docs);
    }
    /**
     * Defines a OPTIONS route on the server.
     * @param {string} path - The route path.
     * @param {(request: Request) => any} callback - The callback function to handle the OPTIONS request.
     */
    options(path, callback, docs = "") {
        hpx.hpxServerOptions(this.serverId, path, (req) => {
            return callback(new Request(req));
        }, docs);
    }
    /**
     * Defines a HEAD route on the server.
     * @param {string} path - The route path.
     * @param {(request: Request) => any} callback - The callback function to handle the HEAD request.
     */
    head(path, callback, docs = "") {
        hpx.hpxServerHead(this.serverId, path, (req) => {
            return callback(new Request(req));
        }, docs);
    }
    /**
     * Defines a TRACE route on the server.
     * @param {string} path - The route path.
     * @param {(request: Request) => any} callback - The callback function to handle the TRACE request.
     */
    trace(path, callback, docs = "") {
        hpx.hpxServerTrace(this.serverId, path, (req) => {
            return callback(new Request(req));
        }, docs);
    }
    /**
     * Defines a COPY route on the server.
     * @param {string} path - The route path.
     * @param {(request: Request) => any} callback - The callback function to handle the TRACE request.
     */
    copy(path, callback, docs = "") {
        hpx.hpxServerCopy(this.serverId, path, (req) => {
            return callback(new Request(req));
        }, docs);
    }
    /**
     * Defines a PURGE route on the server.
     * @param {string} path - The route path.
     * @param {(request: Request) => any} callback - The callback function to handle the TRACE request.
     */
    purge(path, callback, docs = "") {
        hpx.hpxServerPurge(this.serverId, path, (req) => {
            return callback(new Request(req));
        }, docs);
    }
    /**
     * Defines a middleware route on the server.
     * @param {(request: Request) => any} callback - The callback function to handle all HTTP requests.
     */
    middleware(callback) {
        hpx.hpxServerMiddleware(this.serverId, (req) => {
            return callback(new Request(req));
        });
    }
    /**
     * Defines a route on the server that touches when not any routes matched.
     * @param {(request: Request) => any} callback - The callback function to handle the 404 route.
     */
    notfound(callback) {
        hpx.hpxServerNotFound(this.serverId, (req) => {
            return callback(new Request(req));
        });
    }
    /**
    * Defines a WebSocket route on the server.
     * @param {string} path - The route path.
     * @param {(wsClient: WebSocketClient) => any} callback - The callback function to handle WebSocket connections.
    */
    ws(path, callback) {
        let wsClient;
        hpx.hpxServerWebSocket(this.serverId, path, (req) => {
            if (!wsClient) {
                wsClient = new WebSocketClient(req);
            }
            wsClient.setState(req.state);
            callback(wsClient);
        });
    }
    /**
    * Defines a WebSocket route on the server.
     * @param {string} path - The route path.
     * @param {(wsClient: WebSocketClient) => any} callback -
     *        The callback function to handle WebSocket connections.
     *
     * @example
     *  asdasdasdasdasdsa;
     *
     */
    mount(path, other) {
        hpx.hpxServerMount(this.serverId, path, other.serverId);
    }
    /**
     * Serves static files from a directory under a specific path.
     * @param {string} path - The route path.
     * @param {string} directory - The directory to serve static files from.
     */
    static(path, directory) {
        hpx.hpxServerStatic(this.serverId, path, directory);
    }
}
exports.Server = Server;
function newPathParamType(name, pattern, cb) {
    if (typeof pattern === "string") {
        hpx.hpxRegisterPathParamType(name, pattern, cb);
    }
    else {
        hpx.hpxRegisterPathParamType(name, pattern.source, cb);
    }
}
exports.newPathParamType = newPathParamType;
